<html><head><title>Difference of old.txt, new.txt</title></head><body> What is Perl? <br>
<br>
Perl is a  high-level  <b><font color="#FF0000" size="+1" >tool</font></b> &nbsp;<b><font color="#005500" size="+1" >programming language</font></b> with an eclectic heritage written by Larry Wall and a cast of thousands. It derives from the ubiquitous C programming language and to a lesser extent from sed, awk, the Unix shell, and at least a dozen other tools and languages. Perl's process, file, and text manipulation facilities make it particularly well-suited for tasks involving quick <b><font color="#FF0000" size="+1" >writing,</font></b>&nbsp;<b><font color="#005500" size="+1" >prototyping,</font></b> system utilities, software tools, system management tasks, database access, graphical programming, networking, and world wide web programming. These strengths make it especially popular with system <b><font color="#FF0000" size="+1" >admins</font></b>&nbsp;<b><font color="#005500" size="+1" >administrators</font></b> and CGI script authors, but mathematicians, geneticists, journalists, and even managers also use Perl. Maybe you should, too. <br> 
<br>
Who supports Perl? Who develops it? Why is  it   free? <br> <b><font color="#FF0000" size="+1" ><br>This text is taken from perlfaq 1 for demonstration purposes. Toggle &quot;minus&quot; off to view the actual exerpt from the Perlfaq.</font></b>  <br>
<br>
The original culture of the pre-populist Internet and the deeply-held beliefs of Perl's author, Larry Wall, gave rise to the free and open distribution policy of perl. Perl is supported by its users. The core, the standard Perl library, the optional modules, and the documentation you're reading now were all written by volunteers. See the personal note at the end of  the  <b><font color="#FF0000" size="+1" >READIT</font></b> &nbsp;<b><font color="#005500" size="+1" >README</font></b> file in the perl source distribution for more details. See perlhist (new as of 5.005) for Perl's milestone releases. <br> 
<br>
In particular, the core development team (known as the Perl Porters) are a rag-tag band of highly altruistic individuals committed to producing better software for free than you could hope to purchase for money. You may snoop on pending developments via the archives at http://www.xray.mpe.mpg.de/mailing-lists/perl5-porters/ and http://archive.develooper.com/perl5-porters@perl.org/ or the news gateway nntp://nntp.perl.org/perl.perl5.porters or its web interface at http://nntp.perl.org/group/perl.perl5.porters , or read the faq at http://perlhacker.org/p5p-faq , or you can subscribe to the <b><font color="#005500" size="+1" >mailing</font></b> list by sending perl5-porters-request@perl.org a subscription request (an empty message with no subject is fine). <br>
<br>
<b><font color="#005500" size="+1" >While the GNU project includes Perl in its distributions, there's no such thing as &quot;GNU Perl&quot;. Perl is not produced nor maintained by the Free Software Foundation. Perl's licensing terms are also more open than GNU software's tend to be.</font></b> <br>
<br>
You can get commercial support of Perl if you wish, although for most users the informal support will more than suffice. See the answer to &quot;Where can I buy a <b><font color="#005500" size="+1" >commercial</font></b> version of perl?&quot; for more information. </p></body></html>